@@ -71,11 +71,6 @@ options:
       - The username used to authenticate
     required: False
     type: str
-  command:
-    description:
-      - The command to execute in the container
-    required: False
-    type: str
   detach:
     description:
       - Detach from the container after it is created
@@ -203,16 +198,17 @@ EXAMPLES = '''
         action: pull_image
         image: private-registry.example.com:5000/ubuntu
     - name: Create named volume
+      kolla_docker:
         action: create_volume
         name: name_of_volume
     - name: Remove named volume
+      kolla_docker:
         action: remove_volume
         name: name_of_volume
 '''
 
 import json
 import os
-import shlex
 import traceback
 
 import docker
@@ -228,8 +224,6 @@ class DockerWorker(object):
         self.module = module
         self.params = self.module.params
         self.changed = False
-        # Use this to store arguments to pass to exit_json().
-        self.result = {}
 
         # TLS not fully implemented
         # tls_config = self.generate_tls()
@@ -316,7 +310,7 @@ class DockerWorker(object):
             self.compare_volumes_from(container_info) or
             self.compare_environment(container_info) or
             self.compare_container_state(container_info) or
-            self.compare_command(container_info)
+            self.compare_dimensions(container_info)
         )
 
     def compare_ipc_mode(self, container_info):
@@ -453,14 +447,34 @@ class DockerWorker(object):
         if new_state != current_state:
             return True
 
-    def compare_command(self, container_info):
-        new_command = self.params.get('command')
-        if new_command is not None:
-            new_command_split = shlex.split(new_command)
-            new_path = new_command_split[0]
-            new_args = new_command_split[1:]
-            if (new_path != container_info['Path'] or
-                    new_args != container_info['Args']):
+    def compare_dimensions(self, container_info):
+        new_dimensions = self.params.get('dimensions')
+        # NOTE(mgoddard): The names used by Docker are inconsisent between
+        # configuration of a container's resources and the resources in
+        # container_info['HostConfig']. This provides a mapping between the
+        # two.
+        dimension_map = {
+            'mem_limit': 'Memory', 'mem_reservation': 'MemoryReservation',
+            'memswap_limit': 'MemorySwap', 'cpu_period': 'CpuPeriod',
+            'cpu_quota': 'CpuQuota', 'cpu_shares': 'CpuShares',
+            'cpuset_cpus': 'CpusetCpus', 'cpuset_mems': 'CpusetMems',
+            'kernel_memory': 'KernelMemory', 'blkio_weight': 'BlkioWeight'}
+        unsupported = set(new_dimensions.keys()) - \
+            set(dimension_map.keys())
+        if unsupported:
+            self.module.exit_json(
+                failed=True, msg=repr("Unsupported dimensions"),
+                unsupported_dimensions=unsupported)
+        current_dimensions = container_info['HostConfig']
+        for key1, key2 in dimension_map.items():
+            # NOTE(mgoddard): If a resource has been explicitly requested,
+            # check for a match. Otherwise, ensure is is set to the default.
+            if key1 in new_dimensions:
+                if new_dimensions[key1] != current_dimensions[key2]:
+                    return True
+            elif current_dimensions[key2]:
+                # The default values of all currently supported resources are
+                # '' or 0 - both falsey.
                 return True
 
     def parse_image(self):
@@ -577,6 +591,18 @@ class DockerWorker(object):
             'volumes_from': self.params.get('volumes_from')
         }
 
+        if self.params.get('dimensions'):
+            supported = {'cpu_period', 'cpu_quota', 'cpu_shares',
+                         'cpuset_cpus', 'cpuset_mems', 'mem_limit',
+                         'mem_reservation', 'memswap_limit',
+                         'kernel_memory', 'blkio_weight'}
+            unsupported = set(self.params.get('dimensions')) - supported
+            if unsupported:
+                self.module.exit_json(failed=True,
+                                      msg=repr("Unsupported dimensions"),
+                                      unsupported_dimensions=unsupported)
+            options.update(self.params.get('dimensions'))
+
         if self.params.get('restart_policy') in ['on-failure',
                                                  'always',
                                                  'unless-stopped']:
@@ -607,7 +633,6 @@ class DockerWorker(object):
     def build_container_options(self):
         volumes, binds = self.generate_volumes()
         return {
-            'command': self.params.get('command'),
             'detach': self.params.get('detach'),
             'environment': self._format_env_vars(),
             'host_config': self.build_host_config(binds),
@@ -667,22 +692,15 @@ class DockerWorker(object):
             # dict all the time.
             if isinstance(rc, dict):
                 rc = rc['StatusCode']
-            # Include container's return code, standard output and error in the
-            # result.
-            self.result['rc'] = rc
-            self.result['stdout'] = self.dc.logs(self.params.get('name'),
-                                                 stdout=True, stderr=False)
-            self.result['stderr'] = self.dc.logs(self.params.get('name'),
-                                                 stdout=False, stderr=True)
+            if rc != 0:
+                self.module.fail_json(
+                    failed=True,
+                    changed=True,
+                    msg="Container exited with non-zero return code %s" % rc
+                )
             if self.params.get('remove_on_exit'):
                 self.stop_container()
                 self.remove_container()
-            if rc != 0:
-                self.module.fail_json(
-                    changed=True,
-                    msg="Container exited with non-zero return code %s" % rc,
-                    **self.result
-                )
 
     def get_container_env(self):
         name = self.params.get('name')
@@ -774,7 +792,6 @@ def generate_module():
         auth_password=dict(required=False, type='str', no_log=True),
         auth_registry=dict(required=False, type='str'),
         auth_username=dict(required=False, type='str'),
-        command=dict(required=False, type='str'),
         detach=dict(required=False, type='bool', default=True),
         labels=dict(required=False, type='dict', default=dict()),
         name=dict(required=False, type='str'),
@@ -807,6 +824,7 @@ def generate_module():
         tls_cacert=dict(required=False, type='str'),
         volumes=dict(required=False, type='list'),
         volumes_from=dict(required=False, type='list'),
+        dimensions=dict(required=False, type='dict', default=dict()),
         tty=dict(required=False, type='bool', default=False),
     )
     required_if = [
@@ -855,17 +873,16 @@ def generate_module():
 def main():
     module = generate_module()
 
-    dw = None
     try:
         dw = DockerWorker(module)
         # TODO(inc0): We keep it bool to have ansible deal with consistent
         # types. If we ever add method that will have to return some
         # meaningful data, we need to refactor all methods to return dicts.
         result = bool(getattr(dw, module.params.get('action'))())
-        module.exit_json(changed=dw.changed, result=result, **dw.result)
+        module.exit_json(changed=dw.changed, result=result)
     except Exception:
-        module.fail_json(changed=True, msg=repr(traceback.format_exc()),
-                         **getattr(dw, 'result', {}))
+        module.exit_json(failed=True, changed=True,
+                         msg=repr(traceback.format_exc()))
 
 # import module snippets
 from ansible.module_utils.basic import *  # noqa
